#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <time.h>
#include <assert.h>
#include <omp.h>
#include "prime.h"
#include "ompadm.h"

/*
	Copyright (c) 2014 Andrew V. Sutherland
	See LICENSE file for license details
*/

int main (int argc, char *argv[])
{
	prime_table_t tab;
	double begin;
	int threads;
	long k0, m, p, q, r;
	
	if ( argc < 2 ) { puts ("hrsearch k0 [threads]"); return 0; }
	k0 = atol(argv[1]);
	assert ( k0 > 5 );
	if ( argc > 2 ) threads = atoi(argv[2]); else threads = omp_get_max_threads();
	if ( threads <= 0 || threads > omp_get_max_threads() ) threads = omp_get_max_threads();
	printf ("omp threads = %d\n", threads);

	begin = omp_get_wtime();
	m = prime_pi (k0);
	printf ("pi(%ld) = %ld (%0.1fs)\n", k0, m, omp_get_wtime()-begin);
	prime_table_init (tab, 2, m+(k0+1)/2-1);
	printf ("Built prime table from p_%ld=%ld to p_%ld=%ld (%.1fs)\n", tab->si, tab->sp, tab->ti, tab->tp, omp_get_wtime()-begin);

	m = hr_best_m (tab, k0, begin, threads);
	p = prime_table_nth_prime (tab, m+k0/2-1);  q = prime_table_nth_prime (tab, m+(k0+1)/2-1);  r = prime_table_nth_prime (tab, m+1);
	printf ("Minimal m = %ld gives diameter %ld for interval for sequence -p_%ld=%ld,...,-p_%ld=%ld,...,-1,1,...,p_%ld=%ld,...,p_%ld=%ld\n", m, q+p, m+k0/2-1, p, m+1, r, m+1,r, m+(k0+1)/2-1, q);	
	printf ("This result is only heuristic, you now need to run\n    hrverify %ld %ld\n", k0, m);
	printf ("Total elapsed time %.1fs\n", omp_get_wtime()-begin);
	prime_table_clear (tab);
	prime_cleanup();
}
