#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <time.h>
#include <assert.h>
#include <omp.h>
#include "bitmap.h"
#include "prime.h"
#include "ompadm.h"

/*
	Copyright (c) 2014 Andrew V. Sutherland
	See LICENSE file for license details
*/


int main (int argc, char *argv[])
{
	prime_table_t ptab;
	gap_table_t gtab;
	double begin;
	int threads;
	long n, k0, d, m, s, b0, b1;
	
	if ( argc < 2 ) { puts ("schinzelsearch k0 [s threads]"); return 0; }
	k0 = atol(argv[1]);
	assert ( k0 > 13 );
	if ( argc > 2 ) s = atol (argv[2]); else s = 0;
	if ( s ) assert ( s > k0/log(k0) );		// don't let s be too small, we want to avoid large gaps
	if ( argc > 3 ) threads = atoi(argv[3]); else threads = 0;
	if ( threads <= 0 || threads > omp_get_max_threads() ) threads = omp_get_max_threads();
	printf ("omp threads = %d\n", threads);

	if ( ! s ) s = k0;
	if ( (s&1) ) s++;									// make s even
	d = ((((long) (k0*log(k0)+k0))>>6)+1)<<6;

	printf ("initial diameter d=%ld\n", d);

	prime_table_init (ptab, 2, prime_pi(k0)+1);
	
	begin = omp_get_wtime();
	m = schinzel_best_m (ptab, k0, s, &d, begin, threads);
	schinzel_gap_table (gtab, s, d, m, ptab, threads);
	
	n = gap_table_best_start (gtab, k0);
	b0 = gap_table_nth_number (gtab, n);  b1 = gap_table_nth_number (gtab, n+k0-1);
	printf ("Found Schinzel sequence in [%ld,%ld] with k0=%ld using the prime bound p_%ld = %ld, obtaining diameter d=%ld\n",
		   b0, b1, k0, m, prime_table_nth_prime (ptab, m), b1-b0);
	printf ("This results is only heuristic, you now need to run\n    schinzelverify %ld %ld %ld %ld\n", k0, s, d, m);
	gap_table_clear (gtab);
	prime_table_clear (ptab);
	prime_cleanup();

	printf ("Total time %.1fs\n", omp_get_wtime()-begin);
}
